module Inf::AWS
  def self.method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  def self.ec2
    @ec2 ||= Aws::EC2::Client.new
  end

  def self.elb
    @elb ||= Aws::ElasticLoadBalancing::Client.new
  end

  def self.s3
    @s3 ||= Aws::S3::Client.new
  end

  def self.route53
    @route53 ||= Aws::Route53::Client.new
  end

  def self.bootstrap_script(app_name, state_bucket, fleet_name)
    <<BOOTSTRAP_SCRIPT
#!/bin/bash

cd /tmp

cat > bootstrap-env <<EOF
export APP_NAME=#{app_name}
export STATE_BUCKET=#{state_bucket}
export FLEET_NAME=#{fleet_name}
export AWS_REGION=#{ENV['AWS_REGION']}
export JENKINS_PUBLIC_KEY='#{ENV['JENKINS_PUBLIC_KEY']}'
EOF

. ./bootstrap-env

aws s3 cp s3://$STATE_BUCKET/apps/$APP_NAME/bootstrap-instance.sh ./bootstrap.sh
chmod +x bootstrap.sh

( setsid ./bootstrap.sh 2>&1 | setsid logger -t bootstrap ) & disown

BOOTSTRAP_SCRIPT
  end

  def self.launch_load_balancer
    lb_name = "app-lb-#{app_name}"
    cert = ENV['CERT_ARN']

    listeners = [{
      instance_port: 5000,
      instance_protocol: 'HTTP',
      load_balancer_port: 80,
      protocol: 'HTTP'
    }]

    cert && listeners.push(
      instance_port: 5000,
      instance_protocol: 'HTTP',
      load_balancer_port: 443,
      protocol: 'HTTPS',
      ssl_certificate_id: cert
    )

    lb_dns = elb.create_load_balancer(
      load_balancer_name: lb_name,
      subnets: ENV['SUBNET_IDS'].split(','),
      security_groups: [ENV['ELB_SG']],
      scheme: 'internet-facing',
      listeners: listeners
    ).dns_name

    elb.configure_health_check(
      load_balancer_name: lb_name,
      health_check: {
        target: 'HTTP:5000/health_check',
        interval: 10, # seconds
        timeout: 5, # seconds to respond
        unhealthy_threshold: 2, # two consecutive failures
        healthy_threshold: 2
      }
    )

    put_state("apps/#{app_name}/lb-name", lb_name)
    put_state("apps/#{app_name}/lb-dns", lb_dns)

    puts 'Launched load balancer (or it was running)'.blue
  end

  def self.request_spot_fleet(fleet_name='default')
    launch_specifications = ENV['SUBNET_IDS'].split(',').map do |subnet_id|
      ENV['INSTANCE_TYPES'].split(',').map do |type|
        {
          image_id: ENV['AMI'],
          key_name: ENV['KEY_NAME'],
          instance_type: type,
          monitoring: {
            enabled: false,
          },
          network_interfaces: [{
            associate_public_ip_address: true,
            device_index: 0,
            groups: [ENV['SG']],
            subnet_id: subnet_id
          }],
          iam_instance_profile: {
            name: ENV['INSTANCE_PROFILE_NAME'],
          },
          # ebs_optimized: type == 'm1.small' ? false : false,
          weighted_capacity: 1.0,
          user_data: Base64.strict_encode64(bootstrap_script(app_name, state_bucket, fleet_name))
        }
      end
    end.flatten

    tag_specifications = [{
      tags: [{
        key: 'app_name',
        value: app_name
      }, {
        key: 'fleet_name',
        value: fleet_name
      }]
    }]

    resp = ec2.request_spot_fleet(
      spot_fleet_request_config: {
        spot_price: ENV['SPOT_PRICE'],
        target_capacity: ENV['TARGET_CAPACITY'],
        valid_from: Time.now,
        valid_until: Time.parse('1/1/2050'),
        terminate_instances_with_expiration: ENV['TERMINATE_INSTANCES'],
        iam_fleet_role: ENV['FLEET_ROLE'],
        launch_specifications: launch_specifications,
        # tag_specifications: tag_specifications,
        excess_capacity_termination_policy: 'default', # accepts noTermination, default
        allocation_strategy: 'diversified', # accepts lowestPrice, diversified
        type: 'maintain', # accepts request, maintain
        replace_unhealthy_instances: true,
      },
    )

    resp.spot_fleet_request_id.tap do |sfr|
      puts "Launched spot fleet #{fleet_name} (#{sfr})".blue
      put_state("apps/#{app_name}/fleets/#{fleet_name}/sfr", sfr)
    end
  end
end
