class Inf::Fleet < Thor
  def method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  def self.method_missing(m, *args, &block)
    Inf.send(m, *args, &block)
  end

  no_commands do
    def get_sfr(fleet_name='web')
      @sfrs ||= {}
      @sfrs[fleet_name] ||= get_state("apps/#{app_name}/fleets/#{fleet_name}/sfr")
    end
  end

  def self.each_fleet(&block)
    list_state("apps/#{app_name}/fleets") do |key|
      key.match(/fleets\/([^\/]+)/)[1]
    end.uniq.each { |name| block.call name }
  end

  def self.ips(fleet_name='web')
    sfr = new.get_sfr fleet_name

    unless sfr
      puts "No sfr for #{app_name}/#{fleet_name}".red
      puts "Is a fleet launched?"
      exit 1
    end

    instance_ids = ec2.describe_spot_fleet_instances(
      spot_fleet_request_id: sfr
    ).active_instances.map(&:instance_id)

    if instance_ids.empty?
      []
    else
      ec2.describe_instances(
        instance_ids: instance_ids
      ).reservations.map do |it|
        it.instances.first.public_ip_address
      end
    end
  end

  desc 'launch', 'launch app fleet; check env vars in code'
  def launch(fleet_name='web')
    sfr = get_sfr fleet_name

    if sfr
      puts "Kill existing sfr first".red
      exit 1
    end

    Inf::AWS.request_spot_fleet fleet_name

    script = File.expand_path "../../bootstrap-scripts/default-app.sh", __FILE__

    put_state "apps/#{app_name}/bootstrap-instance.sh", File.read(script)
    puts "Don't forget to deploy something".yellow
  end

  desc 'kill', 'kill fleet'
  def kill(fleet_name='web')
    sfr = get_sfr fleet_name

    if sfr
      ec2.cancel_spot_fleet_requests(
        spot_fleet_request_ids: [sfr],
        terminate_instances: true
      )
      puts "Killed #{sfr}".blue
    else
      puts "Did not find a sfr".yellow
    end

    delete_state "apps/#{app_name}/fleets/#{fleet_name}/sfr"
  end

  desc 'rename [NAME] [NEW_NAME]', 'rename fleet'
  def rename(name, new_name)
    sfr = get_sfr new_name

    if sfr
      puts "Fleet #{new_name} already exists".red
      exit 1
    end

    list_state("apps/#{app_name}/fleets/#{name}") do |key|
      leaf = key.split('/').last

      s3.copy_object(
        bucket: state_bucket,
        key: "apps/#{app_name}/fleets/#{new_name}/#{leaf}",
        copy_source: "#{state_bucket}/#{key}"
      )

      delete_state key
    end
  end

  desc 'nodes', 'list public ips of nodes in fleet'
  def nodes(fleet_name='web')
    ips = Inf::Fleet.ips fleet_name

    if ips.any?
      puts ips.join("\n")
    else
      puts "No active instances in fleet"
    end
  end

  desc 'ssh', 'ssh to a node in the fleet'
  def ssh(fleet_name='web')
    ips = Inf::Fleet.ips fleet_name

    if ips.empty?
      puts "No instances in fleet #{fleet_name}".red
    else
      cmd = "#{SSH_CMD} ubuntu@#{ips.first}"

      puts cmd.blue
      Kernel.exec cmd
    end
  end

  desc 'scale FLEET_NAME COUNT', 'scales the fleet'
  def scale(fleet_name, count)
    sfr = get_sfr fleet_name

    ec2.modify_spot_fleet_request({
      spot_fleet_request_id: sfr,
      target_capacity: count.to_i,
      excess_capacity_termination_policy: "default"
    })
  end

  desc 'status', 'describes the fleet'
  def status(fleet_name='web', indent='')
    nodes = Inf::Fleet.ips fleet_name
    sfr = get_state("apps/#{app_name}/fleets/#{fleet_name}/sfr")

    sfr_status = ec2.describe_spot_fleet_requests(
      spot_fleet_request_ids: [sfr]
    ).spot_fleet_request_configs[0].spot_fleet_request_state

    if nodes.any?
      puts "Fleet #{fleet_name} has #{nodes.count} nodes (#{nodes.join ', '})".blue
    elsif sfr
      puts "Fleet #{fleet_name} (#{sfr}: #{sfr_status}) has no nodes!".red
    else
      puts "No fleet #{fleet_name} is launched".yelow
    end
  end

  desc 'list', 'lists fleets'
  def list
    Inf::Fleet.each_fleet { |name| puts name }
  end

  desc 'status_all', 'runs `status` for all fleets'
  def status_all
    Inf::Fleet.each_fleet { |name| status name }
  end
end
